/*  pack: an install wrapper that logs all files installed for easy
	  uninstalling.
    Copyright (C) 1998 Matt Perry <guy@somewhere.linuxos.org>.   

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _pack_h
#define _pack_h

#define PACK_VER_MAJOR 1
#define PACK_VER_MINOR 1
#define PACK_VER_PATCH 5

#define MEM_BLOCK	256

#define strmat(a, b)	(strcmp(a, b) == 0)
#define mnew(t, c)	((t*)calloc(1, sizeof(t)*c))
#define mrenew(t, p, c)	((t*)realloc(p, sizeof(t)*c))
#define mremove(argc, argv, i) {\
	for (int q = i; argv[q]; ++q)\
		argv[q] = argv[q+1];\
	--argc;\
}

template<class T>
class Buffer {
public:
	T*		data;
	long	len;
	long	size;
public:
    Buffer();
	void addlen(int);
};

/* Classes
 */
class PackObj {
public:
	int		argc;
	char**	argv;
	char*	cmdarg;
	char*	packcur;

	Buffer<char>	chksum;

#define FLAG_VERBOSE		(1<<0)
#define FLAG_DIR_ARG		(1<<1)
#define FLAG_PROMPT_ERASE	(1<<2)
#define FLAG_CHKSUM			(1<<3)
#define V(x)	if ((flags & FLAG_VERBOSE)) {x}
	long	flags;

public:
	PackObj(int, char**);

	void	cmd_new();
	void	cmd_done();
	void	cmd_list();
	void	cmd_erase();
	void	cmd_install();

	void	install_dirs(char**, char**);
	void	install_files(char**, char**);
	void	add_entry(char*);

	void    write_chksum();
	void	verify_chksum();

	char*	get_packcur(char* = 0);
	int		execute();
};

extern char gpath[];
class Path {
public:
	char*	pathstr;
public:
	Path(char*, char*, char* = gpath);
	Path(char*);
	operator char*()
		{ return pathstr; }

	char*	filename();
	char*	lastdir();
//	char*	demangle(); NOT USED
};

/* Functions
 */
extern "C" {
extern char* strdup(const char*);
extern FILE* popen(const char*, const char*);
extern int pclose(FILE*);
}

int	strpre(char*, char*);

void	needhelp(void);
void	version(void);
void	usage(void);

void	process_args(int &, char** &);

#ifdef _mainfile
#define EXTERN
#else
#define EXTERN extern
#endif

EXTERN char*	progname;
EXTERN char	gpath[4096];
EXTERN char*	install_path;
EXTERN char*	chksum_path;
EXTERN char*	db_dir;
EXTERN int	argind;
EXTERN int	packmode;
EXTERN PackObj	*p_obj;

#endif /* _pack_h */
